/*
 * $Id: data.c 44.7 1999/09/16 09:20:27 olsen Exp olsen $
 *
 * :ts=8
 *
 * COPYRIGHT:
 *
 *   Unless otherwise noted, all files are Copyright (c) 1999 Amiga, Inc.
 *   All rights reserved.
 *
 * DISCLAIMER:
 *
 *   This software is provided "as is". No representations or warranties
 *   are made with respect to the accuracy, reliability, performance,
 *   currentness, or operation of this software, and all use is at your
 *   own risk. Neither Amiga nor the authors assume any responsibility
 *   or liability whatsoever with respect to your use of this software.
 *
 */

#include "global.h"

/****************************************************************************/

#include "config.h"

/****************************************************************************/

struct Library * SysBase;
struct Library * UtilityBase;

/****************************************************************************/

struct PrinterExtendedData * PED;
struct PrinterData * PD;

/****************************************************************************/

PWRITE_T PWrite;
PBOTHREADY_T PBothReady;

/****************************************************************************/

STRPTR CommandTable[aRAW+1] =
{
	"\375\033E\375",	/* 00 aRIS reset			*/
	"\377",			/* 01 aRIN initialize			*/
	"\012",			/* 02 aIND linefeed			*/
	"\015\012",		/* 03 aNEL CRLF				*/
	"\033&a-1R",		/* 04 aRI reverse LF			*/

	"\033&d@\033(sbS",	/* 05 aSGR0 normal char set		*/
	"\033(s1S",		/* 06 aSGR3 italics on			*/
	"\033(sS",		/* 07 aSGR23 italics off		*/
	"\033&dD",		/* 08 aSGR4 underline on		*/
	"\033&d@",		/* 09 aSGR24 underline off		*/
	"\033(s5B",		/* 10 aSGR1 boldface on			*/
	"\033(sB",		/* 11 aSGR22 boldface off		*/
	"\377",			/* 12 aSFC set foreground color		*/
	"\377",			/* 13 aSBC set background color		*/

	"\033(s10h1T",		/* 14 aSHORP0 normal pitch		*/
	"\033(s12h2T",		/* 15 aSHORP2 elite on			*/
	"\033(s10h1T",		/* 16 aSHORP1 elite off			*/
	"\033(s15H",		/* 17 aSHORP4 condensed fine on		*/
	"\033(s10H",		/* 18 aSHORP3 condensed fine off	*/
	"\377",			/* 19 aSHORP6 enlarge on		*/
	"\377",			/* 20 aSHORP5 enlarge off		*/

	"\033(s7B",		/* 21 aDEN6 shadow print on		*/
	"\033(sB",		/* 22 aDEN5 shadow print off		*/
	"\033(s3B",		/* 23 aDEN4 double strike on		*/
	"\033(sB",		/* 24 aDEN3 double strike off		*/
	"\033(s2Q",		/* 25 aDEN2 NLQ on			*/
	"\033(s0Q",		/* 26 aDEN1 NLQ off			*/

	"\377",			/* 27 aSUS2 superscript on		*/
	"\377",			/* 28 aSUS1 superscript off		*/
	"\377",			/* 29 aSUS4 subscript on		*/
	"\377",			/* 30 aSUS3 subscript off		*/
	"\377",			/* 31 aSUS0 normalize the line		*/
	"\033&a-.5R",		/* 32 aPLU partial line up		*/
	"\033=",		/* 33 aPLD partial line down		*/

	"\033(s3T",		/* 34 aFNT0 Typeface 0			*/
	"\033(s0T",		/* 35 aFNT1 Typeface 1			*/
	"\033(s1T",		/* 36 aFNT2 Typeface 2			*/
	"\033(s2T",		/* 37 aFNT3 Typeface 3			*/
	"\033(s4T",		/* 38 aFNT4 Typeface 4			*/
	"\033(s5T",		/* 39 aFNT5 Typeface 5			*/
	"\033(s6T",		/* 40 aFNT6 Typeface 6			*/
	"\033(s7T",		/* 41 aFNT7 Typeface 7			*/
	"\033(s8T",		/* 42 aFNT8 Typeface 8			*/
	"\033(s9T",		/* 43 aFNT9 Typeface 9			*/
	"\033(s10T",		/* 44 aFNT10 Typeface 10		*/
                              
	"\033(s1P",		/* 45 aPROP2 proportional on		*/
	"\033(sP",		/* 46 aPROP1 proportional off		*/
	"\033(sP",		/* 47 aPROP0 proportional clear		*/
	"\377",			/* 48 aTSS set proportional offset	*/
	"\377",			/* 49 aJFY5 auto left justify		*/
	"\377",			/* 50 aJFY7 auto right justify		*/
	"\377",			/* 51 aJFY6 auto full jusitfy		*/
	"\377",			/* 52 aJFY0 auto jusity off		*/
	"\377",			/* 53 aJFY3 letter space		*/
	"\377",			/* 54 aJFY1 word fill			*/

	"\033&l8D",		/* 55 aVERP0 1/8" line spacing		*/
	"\033&l6D",		/* 56 aVERP1 1/6" line spacing		*/
	"\377",			/* 57 aSLPP set form length		*/
	"\033&l1L",		/* 58 aPERF perf skip n (n > 0)		*/
	"\033&lL",		/* 59 aPERF0 perf skip off		*/
                        
	"\377",			/* 60 aLMS set left margin		*/
	"\377",			/* 61 aRMS set right margin		*/
	"\377",			/* 62 aTMS set top margin		*/
	"\377",			/* 63 aBMS set bottom margin		*/
	"\377",			/* 64 aSTBM set T&B margins		*/
	"\377",			/* 65 aSLRM set L&R margins		*/
	"\0339\015",		/* 66 aCAM clear margins		*/

	"\377",			/* 67 aHTS set horiz tab		*/
	"\377",			/* 68 aVTS set vert tab			*/
	"\377",			/* 69 aTBC0 clear horiz tab		*/
	"\377",			/* 70 aTBC3 clear all horiz tabs	*/
	"\377",			/* 71 aTBC1 clear vert tab		*/
	"\377",			/* 72 aTBC4 clear all vert tabs		*/
	"\377",			/* 73 aTBCALL clear all h & v tabs	*/
	"\377",			/* 74 aTBSALL set default tabs		*/

	"\377",			/* 75 aEXTEND extended commands		*/
	"\377"			/* 76 aRAW next 'n' chars are raw	*/
};

/****************************************************************************/

STRPTR ExtendedCharTable[96] =
{
	" ", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", ""
};

/****************************************************************************/

#if CONFIG_SUPPORTS_COLOR
#define DENSITY_INFO_TAG PRTA_DensityInfo
#else 
#define DENSITY_INFO_TAG TAG_IGNORE
#endif /* CONFIG_SUPPORTS_COLOR */

/****************************************************************************/

/* Filled in by locale.c */
STRPTR DensityNames[8];

/****************************************************************************/

struct TagItem PropertyTagList[] =
{
	{ PRTA_8BitGuns,	TRUE				},
	{ PRTA_FloydDithering,	TRUE				},
	{ DENSITY_INFO_TAG,	(ULONG)DensityNames		},
	{ PRTA_MixBWColor,	CONFIG_COLOR_BW_SAME_RESOLUTION	},
	{ PRTA_LeftBorder,	166				},	/* 50 dots at 300 DPI */
	{ PRTA_TopBorder,	166				},	/* 50 dots at 300 DPI */

	{ TAG_DONE }
};

/****************************************************************************/

#if CONFIG_SUPPORTS_COLOR

/****************************************************************************/

UBYTE GammaTables[15][256] =
{
	/* Gamma = 1.1 */
	{
		0x00,0x01,0x01,0x02,0x03,0x03,0x04,0x05,0x06,0x06,0x07,0x08,0x09,0x0A,0x0A,0x0B,
		0x0C,0x0D,0x0E,0x0F,0x10,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x16,0x17,0x18,0x19,
		0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x1F,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,
		0x29,0x2A,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,
		0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,
		0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x52,0x53,0x54,0x55,0x56,
		0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,0x60,0x61,0x62,0x63,0x64,0x65,0x66,
		0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70,0x71,0x72,0x73,0x74,0x75,0x76,
		0x77,0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,
		0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,0x90,0x91,0x92,0x93,0x95,0x96,0x97,0x98,
		0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F,0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA9,
		0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xBA,
		0xBB,0xBC,0xBD,0xBE,0xBF,0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC9,0xCA,0xCB,
		0xCC,0xCD,0xCE,0xCF,0xD0,0xD1,0xD2,0xD3,0xD4,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,
		0xDD,0xDE,0xDF,0xE0,0xE1,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,
		0xEF,0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
	},

	/* Gamma = 1.2 */
	{
		0x00,0x00,0x01,0x01,0x02,0x02,0x03,0x03,0x04,0x05,0x05,0x06,0x07,0x07,0x08,0x09,
		0x09,0x0A,0x0B,0x0B,0x0C,0x0D,0x0D,0x0E,0x0F,0x10,0x10,0x11,0x12,0x13,0x14,0x14,
		0x15,0x16,0x17,0x18,0x18,0x19,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,
		0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,
		0x31,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3E,
		0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,
		0x4F,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,
		0x5F,0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,
		0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,
		0x80,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x90,0x91,
		0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9B,0x9C,0x9D,0x9E,0x9F,0xA0,0xA1,0xA2,
		0xA3,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAF,0xB0,0xB1,0xB2,0xB3,0xB4,
		0xB5,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBF,0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC7,
		0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCF,0xD0,0xD1,0xD2,0xD3,0xD4,0xD6,0xD7,0xD8,0xD9,
		0xDA,0xDB,0xDD,0xDE,0xDF,0xE0,0xE1,0xE2,0xE4,0xE5,0xE6,0xE7,0xE8,0xEA,0xEB,0xEC,
		0xED,0xEE,0xEF,0xF1,0xF2,0xF3,0xF4,0xF5,0xF7,0xF8,0xF9,0xFA,0xFB,0xFD,0xFE,0xFF
	},

	/* Gamma = 1.3 */
	{
		0x00,0x00,0x00,0x01,0x01,0x02,0x02,0x02,0x03,0x03,0x04,0x04,0x05,0x05,0x06,0x06,
		0x07,0x08,0x08,0x09,0x09,0x0A,0x0B,0x0B,0x0C,0x0C,0x0D,0x0E,0x0E,0x0F,0x10,0x10,
		0x11,0x12,0x13,0x13,0x14,0x15,0x15,0x16,0x17,0x18,0x18,0x19,0x1A,0x1B,0x1C,0x1C,
		0x1D,0x1E,0x1F,0x1F,0x20,0x21,0x22,0x23,0x24,0x24,0x25,0x26,0x27,0x28,0x29,0x29,
		0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,
		0x39,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,
		0x48,0x49,0x4A,0x4B,0x4C,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x52,0x53,0x54,0x55,0x56,
		0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,
		0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,
		0x79,0x7A,0x7C,0x7D,0x7E,0x7F,0x80,0x81,0x82,0x83,0x84,0x85,0x87,0x88,0x89,0x8A,
		0x8B,0x8C,0x8D,0x8F,0x90,0x91,0x92,0x93,0x94,0x95,0x97,0x98,0x99,0x9A,0x9B,0x9C,
		0x9D,0x9F,0xA0,0xA1,0xA2,0xA3,0xA4,0xA6,0xA7,0xA8,0xA9,0xAA,0xAC,0xAD,0xAE,0xAF,
		0xB0,0xB2,0xB3,0xB4,0xB5,0xB6,0xB8,0xB9,0xBA,0xBB,0xBC,0xBE,0xBF,0xC0,0xC1,0xC2,
		0xC4,0xC5,0xC6,0xC7,0xC9,0xCA,0xCB,0xCC,0xCE,0xCF,0xD0,0xD1,0xD2,0xD4,0xD5,0xD6,
		0xD7,0xD9,0xDA,0xDB,0xDC,0xDE,0xDF,0xE0,0xE2,0xE3,0xE4,0xE5,0xE7,0xE8,0xE9,0xEA,
		0xEC,0xED,0xEE,0xF0,0xF1,0xF2,0xF3,0xF5,0xF6,0xF7,0xF9,0xFA,0xFB,0xFC,0xFE,0xFF
	},

	/* Gamma = 1.4 */
	{
		0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x03,0x03,0x04,0x04,0x04,0x05,
		0x05,0x06,0x06,0x07,0x07,0x08,0x08,0x09,0x09,0x0A,0x0A,0x0B,0x0C,0x0C,0x0D,0x0D,
		0x0E,0x0F,0x0F,0x10,0x10,0x11,0x12,0x12,0x13,0x14,0x14,0x15,0x16,0x16,0x17,0x18,
		0x19,0x19,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1F,0x1F,0x20,0x21,0x22,0x22,0x23,0x24,
		0x25,0x26,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,0x31,0x31,
		0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,
		0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,
		0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,0x60,
		0x61,0x62,0x63,0x64,0x65,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70,0x71,
		0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x7C,0x7D,0x7E,0x7F,0x80,0x81,0x82,0x84,
		0x85,0x86,0x87,0x88,0x89,0x8B,0x8C,0x8D,0x8E,0x8F,0x91,0x92,0x93,0x94,0x95,0x97,
		0x98,0x99,0x9A,0x9B,0x9D,0x9E,0x9F,0xA0,0xA1,0xA3,0xA4,0xA5,0xA6,0xA8,0xA9,0xAA,
		0xAB,0xAD,0xAE,0xAF,0xB0,0xB2,0xB3,0xB4,0xB5,0xB7,0xB8,0xB9,0xBB,0xBC,0xBD,0xBE,
		0xC0,0xC1,0xC2,0xC4,0xC5,0xC6,0xC8,0xC9,0xCA,0xCB,0xCD,0xCE,0xCF,0xD1,0xD2,0xD3,
		0xD5,0xD6,0xD7,0xD9,0xDA,0xDB,0xDD,0xDE,0xDF,0xE1,0xE2,0xE3,0xE5,0xE6,0xE8,0xE9,
		0xEA,0xEC,0xED,0xEE,0xF0,0xF1,0xF2,0xF4,0xF5,0xF7,0xF8,0xF9,0xFB,0xFC,0xFE,0xFF
	},

	/* Gamma = 1.5 */
	{
		0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x03,0x03,0x03,0x04,
		0x04,0x04,0x05,0x05,0x06,0x06,0x06,0x07,0x07,0x08,0x08,0x09,0x09,0x0A,0x0A,0x0B,
		0x0B,0x0C,0x0C,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x12,0x12,0x13,0x14,0x14,
		0x15,0x15,0x16,0x17,0x17,0x18,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1F,0x1F,
		0x20,0x21,0x22,0x22,0x23,0x24,0x25,0x25,0x26,0x27,0x28,0x29,0x29,0x2A,0x2B,0x2C,
		0x2D,0x2E,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x35,0x36,0x37,0x38,0x39,0x3A,
		0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,
		0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x58,0x59,0x5A,
		0x5B,0x5C,0x5D,0x5E,0x5F,0x60,0x61,0x62,0x63,0x64,0x66,0x67,0x68,0x69,0x6A,0x6B,
		0x6C,0x6D,0x6E,0x70,0x71,0x72,0x73,0x74,0x75,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7E,
		0x7F,0x80,0x81,0x82,0x84,0x85,0x86,0x87,0x88,0x8A,0x8B,0x8C,0x8D,0x8E,0x90,0x91,
		0x92,0x93,0x95,0x96,0x97,0x98,0x9A,0x9B,0x9C,0x9E,0x9F,0xA0,0xA1,0xA3,0xA4,0xA5,
		0xA7,0xA8,0xA9,0xAB,0xAC,0xAD,0xAE,0xB0,0xB1,0xB2,0xB4,0xB5,0xB6,0xB8,0xB9,0xBB,
		0xBC,0xBD,0xBF,0xC0,0xC1,0xC3,0xC4,0xC5,0xC7,0xC8,0xCA,0xCB,0xCC,0xCE,0xCF,0xD1,
		0xD2,0xD3,0xD5,0xD6,0xD8,0xD9,0xDA,0xDC,0xDD,0xDF,0xE0,0xE2,0xE3,0xE4,0xE6,0xE7,
		0xE9,0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5,0xF6,0xF8,0xF9,0xFB,0xFC,0xFE,0xFF
	},

	/* Gamma = 1.6 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x03,
		0x03,0x03,0x04,0x04,0x04,0x05,0x05,0x05,0x06,0x06,0x07,0x07,0x07,0x08,0x08,0x09,
		0x09,0x0A,0x0A,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,
		0x12,0x12,0x13,0x13,0x14,0x15,0x15,0x16,0x17,0x17,0x18,0x19,0x19,0x1A,0x1B,0x1B,
		0x1C,0x1D,0x1D,0x1E,0x1F,0x1F,0x20,0x21,0x22,0x22,0x23,0x24,0x25,0x26,0x26,0x27,
		0x28,0x29,0x2A,0x2A,0x2B,0x2C,0x2D,0x2E,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,
		0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x40,0x41,0x42,0x43,
		0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x53,0x54,
		0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,0x61,0x62,0x63,0x64,0x65,
		0x66,0x67,0x68,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x71,0x72,0x73,0x74,0x75,0x77,0x78,
		0x79,0x7A,0x7B,0x7D,0x7E,0x7F,0x80,0x82,0x83,0x84,0x85,0x87,0x88,0x89,0x8A,0x8C,
		0x8D,0x8E,0x8F,0x91,0x92,0x93,0x95,0x96,0x97,0x99,0x9A,0x9B,0x9D,0x9E,0x9F,0xA1,
		0xA2,0xA3,0xA5,0xA6,0xA7,0xA9,0xAA,0xAB,0xAD,0xAE,0xB0,0xB1,0xB2,0xB4,0xB5,0xB7,
		0xB8,0xB9,0xBB,0xBC,0xBE,0xBF,0xC1,0xC2,0xC4,0xC5,0xC6,0xC8,0xC9,0xCB,0xCC,0xCE,
		0xCF,0xD1,0xD2,0xD4,0xD5,0xD7,0xD8,0xDA,0xDB,0xDD,0xDE,0xE0,0xE1,0xE3,0xE4,0xE6,
		0xE7,0xE9,0xEB,0xEC,0xEE,0xEF,0xF1,0xF2,0xF4,0xF5,0xF7,0xF9,0xFA,0xFC,0xFD,0xFF
	},

	/* Gamma = 1.7 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,
		0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x05,0x05,0x05,0x06,0x06,0x06,0x07,0x07,
		0x07,0x08,0x08,0x09,0x09,0x0A,0x0A,0x0A,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0E,0x0E,
		0x0F,0x0F,0x10,0x11,0x11,0x12,0x12,0x13,0x13,0x14,0x15,0x15,0x16,0x16,0x17,0x18,
		0x18,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,0x20,0x21,0x21,0x22,0x23,
		0x24,0x24,0x25,0x26,0x27,0x27,0x28,0x29,0x2A,0x2B,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,
		0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,
		0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,
		0x4F,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,
		0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x69,0x6A,0x6B,0x6C,0x6D,0x6F,0x70,0x71,0x72,
		0x73,0x75,0x76,0x77,0x78,0x7A,0x7B,0x7C,0x7D,0x7F,0x80,0x81,0x83,0x84,0x85,0x86,
		0x88,0x89,0x8A,0x8C,0x8D,0x8E,0x90,0x91,0x92,0x94,0x95,0x97,0x98,0x99,0x9B,0x9C,
		0x9D,0x9F,0xA0,0xA2,0xA3,0xA4,0xA6,0xA7,0xA9,0xAA,0xAC,0xAD,0xAE,0xB0,0xB1,0xB3,
		0xB4,0xB6,0xB7,0xB9,0xBA,0xBC,0xBD,0xBF,0xC0,0xC2,0xC3,0xC5,0xC6,0xC8,0xC9,0xCB,
		0xCD,0xCE,0xD0,0xD1,0xD3,0xD4,0xD6,0xD8,0xD9,0xDB,0xDC,0xDE,0xE0,0xE1,0xE3,0xE4,
		0xE6,0xE8,0xE9,0xEB,0xED,0xEE,0xF0,0xF2,0xF3,0xF5,0xF7,0xF8,0xFA,0xFC,0xFD,0xFF
	},

	/* Gamma = 1.8 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,
		0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x06,
		0x06,0x06,0x07,0x07,0x08,0x08,0x08,0x09,0x09,0x0A,0x0A,0x0A,0x0B,0x0B,0x0C,0x0C,
		0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x11,0x12,0x12,0x13,0x13,0x14,0x15,
		0x15,0x16,0x16,0x17,0x18,0x18,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,
		0x20,0x20,0x21,0x22,0x23,0x23,0x24,0x25,0x26,0x26,0x27,0x28,0x29,0x29,0x2A,0x2B,
		0x2C,0x2D,0x2E,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x35,0x36,0x37,0x38,0x39,
		0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,
		0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x52,0x53,0x54,0x56,0x57,0x58,0x59,0x5A,
		0x5B,0x5C,0x5D,0x5F,0x60,0x61,0x62,0x63,0x64,0x66,0x67,0x68,0x69,0x6B,0x6C,0x6D,
		0x6E,0x6F,0x71,0x72,0x73,0x74,0x76,0x77,0x78,0x7A,0x7B,0x7C,0x7E,0x7F,0x80,0x81,
		0x83,0x84,0x86,0x87,0x88,0x8A,0x8B,0x8C,0x8E,0x8F,0x91,0x92,0x93,0x95,0x96,0x98,
		0x99,0x9A,0x9C,0x9D,0x9F,0xA0,0xA2,0xA3,0xA5,0xA6,0xA8,0xA9,0xAB,0xAC,0xAE,0xAF,
		0xB1,0xB2,0xB4,0xB5,0xB7,0xB8,0xBA,0xBC,0xBD,0xBF,0xC0,0xC2,0xC3,0xC5,0xC7,0xC8,
		0xCA,0xCC,0xCD,0xCF,0xD0,0xD2,0xD4,0xD5,0xD7,0xD9,0xDA,0xDC,0xDE,0xE0,0xE1,0xE3,
		0xE5,0xE6,0xE8,0xEA,0xEC,0xED,0xEF,0xF1,0xF3,0xF4,0xF6,0xF8,0xFA,0xFB,0xFD,0xFF
	},

	/* Gamma = 1.9 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,
		0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,
		0x05,0x05,0x06,0x06,0x06,0x07,0x07,0x07,0x08,0x08,0x08,0x09,0x09,0x09,0x0A,0x0A,
		0x0B,0x0B,0x0C,0x0C,0x0C,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x11,0x12,
		0x12,0x13,0x14,0x14,0x15,0x15,0x16,0x16,0x17,0x18,0x18,0x19,0x1A,0x1A,0x1B,0x1C,
		0x1C,0x1D,0x1E,0x1E,0x1F,0x20,0x20,0x21,0x22,0x23,0x23,0x24,0x25,0x26,0x26,0x27,
		0x28,0x29,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,
		0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,
		0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x51,0x52,0x53,0x54,0x55,
		0x56,0x57,0x58,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,0x61,0x62,0x63,0x64,0x65,0x67,0x68,
		0x69,0x6A,0x6C,0x6D,0x6E,0x70,0x71,0x72,0x73,0x75,0x76,0x77,0x79,0x7A,0x7B,0x7D,
		0x7E,0x7F,0x81,0x82,0x84,0x85,0x86,0x88,0x89,0x8B,0x8C,0x8D,0x8F,0x90,0x92,0x93,
		0x95,0x96,0x98,0x99,0x9B,0x9C,0x9E,0x9F,0xA1,0xA2,0xA4,0xA5,0xA7,0xA8,0xAA,0xAC,
		0xAD,0xAF,0xB0,0xB2,0xB4,0xB5,0xB7,0xB8,0xBA,0xBC,0xBD,0xBF,0xC1,0xC2,0xC4,0xC6,
		0xC7,0xC9,0xCB,0xCC,0xCE,0xD0,0xD2,0xD3,0xD5,0xD7,0xD9,0xDA,0xDC,0xDE,0xE0,0xE1,
		0xE3,0xE5,0xE7,0xE9,0xEB,0xEC,0xEE,0xF0,0xF2,0xF4,0xF6,0xF7,0xF9,0xFB,0xFD,0xFF
	},

	/* Gamma = 2.0 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,
		0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,
		0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x07,0x07,0x07,0x08,0x08,0x08,0x09,
		0x09,0x09,0x0A,0x0A,0x0B,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,
		0x10,0x11,0x11,0x12,0x12,0x13,0x13,0x14,0x14,0x15,0x15,0x16,0x17,0x17,0x18,0x18,
		0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,0x20,0x20,0x21,0x22,0x23,0x23,
		0x24,0x25,0x26,0x26,0x27,0x28,0x29,0x2A,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,0x2F,0x30,
		0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
		0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,
		0x51,0x52,0x54,0x55,0x56,0x57,0x58,0x59,0x5B,0x5C,0x5D,0x5E,0x5F,0x61,0x62,0x63,
		0x64,0x66,0x67,0x68,0x69,0x6B,0x6C,0x6D,0x6F,0x70,0x71,0x73,0x74,0x75,0x77,0x78,
		0x79,0x7B,0x7C,0x7E,0x7F,0x80,0x82,0x83,0x85,0x86,0x88,0x89,0x8B,0x8C,0x8E,0x8F,
		0x91,0x92,0x94,0x95,0x97,0x98,0x9A,0x9B,0x9D,0x9E,0xA0,0xA2,0xA3,0xA5,0xA6,0xA8,
		0xAA,0xAB,0xAD,0xAF,0xB0,0xB2,0xB4,0xB5,0xB7,0xB9,0xBA,0xBC,0xBE,0xC0,0xC1,0xC3,
		0xC5,0xC7,0xC8,0xCA,0xCC,0xCE,0xCF,0xD1,0xD3,0xD5,0xD7,0xD9,0xDA,0xDC,0xDE,0xE0,
		0xE2,0xE4,0xE6,0xE8,0xE9,0xEB,0xED,0xEF,0xF1,0xF3,0xF5,0xF7,0xF9,0xFB,0xFD,0xFF
	},

	/* Gamma = 2.1 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,
		0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,
		0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x07,0x07,0x07,
		0x08,0x08,0x08,0x09,0x09,0x09,0x0A,0x0A,0x0B,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0E,
		0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x11,0x12,0x12,0x13,0x14,0x14,0x15,0x15,0x16,
		0x16,0x17,0x18,0x18,0x19,0x19,0x1A,0x1B,0x1B,0x1C,0x1D,0x1D,0x1E,0x1F,0x1F,0x20,
		0x21,0x21,0x22,0x23,0x24,0x24,0x25,0x26,0x27,0x28,0x28,0x29,0x2A,0x2B,0x2C,0x2C,
		0x2D,0x2E,0x2F,0x30,0x31,0x32,0x33,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,
		0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4B,0x4C,
		0x4D,0x4E,0x4F,0x50,0x51,0x53,0x54,0x55,0x56,0x57,0x58,0x5A,0x5B,0x5C,0x5D,0x5F,
		0x60,0x61,0x62,0x64,0x65,0x66,0x68,0x69,0x6A,0x6B,0x6D,0x6E,0x70,0x71,0x72,0x74,
		0x75,0x76,0x78,0x79,0x7B,0x7C,0x7E,0x7F,0x81,0x82,0x83,0x85,0x86,0x88,0x89,0x8B,
		0x8D,0x8E,0x90,0x91,0x93,0x94,0x96,0x97,0x99,0x9B,0x9C,0x9E,0xA0,0xA1,0xA3,0xA5,
		0xA6,0xA8,0xAA,0xAB,0xAD,0xAF,0xB0,0xB2,0xB4,0xB6,0xB7,0xB9,0xBB,0xBD,0xBF,0xC0,
		0xC2,0xC4,0xC6,0xC8,0xCA,0xCB,0xCD,0xCF,0xD1,0xD3,0xD5,0xD7,0xD9,0xDB,0xDD,0xDF,
		0xE1,0xE2,0xE4,0xE6,0xE8,0xEA,0xEC,0xEE,0xF1,0xF3,0xF5,0xF7,0xF9,0xFB,0xFD,0xFF
	},

	/* Gamma = 2.2 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
		0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
		0x03,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,
		0x06,0x07,0x07,0x07,0x08,0x08,0x08,0x09,0x09,0x09,0x0A,0x0A,0x0B,0x0B,0x0B,0x0C,
		0x0C,0x0D,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x11,0x12,0x12,0x13,0x13,
		0x14,0x14,0x15,0x16,0x16,0x17,0x17,0x18,0x19,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,
		0x1E,0x1E,0x1F,0x20,0x21,0x21,0x22,0x23,0x23,0x24,0x25,0x26,0x27,0x27,0x28,0x29,
		0x2A,0x2B,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,0x31,0x31,0x32,0x33,0x34,0x35,0x36,0x37,
		0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,
		0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x51,0x52,0x53,0x54,0x55,0x57,0x58,0x59,0x5A,
		0x5B,0x5D,0x5E,0x5F,0x61,0x62,0x63,0x64,0x66,0x67,0x69,0x6A,0x6B,0x6D,0x6E,0x6F,
		0x71,0x72,0x74,0x75,0x77,0x78,0x79,0x7B,0x7C,0x7E,0x7F,0x81,0x82,0x84,0x85,0x87,
		0x89,0x8A,0x8C,0x8D,0x8F,0x91,0x92,0x94,0x95,0x97,0x99,0x9A,0x9C,0x9E,0x9F,0xA1,
		0xA3,0xA5,0xA6,0xA8,0xAA,0xAC,0xAD,0xAF,0xB1,0xB3,0xB5,0xB6,0xB8,0xBA,0xBC,0xBE,
		0xC0,0xC2,0xC4,0xC5,0xC7,0xC9,0xCB,0xCD,0xCF,0xD1,0xD3,0xD5,0xD7,0xD9,0xDB,0xDD,
		0xDF,0xE1,0xE3,0xE5,0xE7,0xEA,0xEC,0xEE,0xF0,0xF2,0xF4,0xF6,0xF8,0xFB,0xFD,0xFF
	},

	/* Gamma = 2.3 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,
		0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x04,0x05,0x05,0x05,
		0x05,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,0x08,0x08,0x09,0x09,0x0A,0x0A,0x0A,
		0x0B,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x10,0x10,0x11,0x11,
		0x12,0x12,0x13,0x13,0x14,0x14,0x15,0x15,0x16,0x17,0x17,0x18,0x18,0x19,0x1A,0x1A,
		0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,0x20,0x20,0x21,0x22,0x23,0x23,0x24,0x25,0x26,
		0x26,0x27,0x28,0x29,0x2A,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,0x31,0x31,0x32,0x33,
		0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,0x40,0x41,0x42,0x43,
		0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4E,0x4F,0x50,0x51,0x52,0x54,0x55,0x56,
		0x57,0x59,0x5A,0x5B,0x5C,0x5E,0x5F,0x60,0x62,0x63,0x64,0x66,0x67,0x68,0x6A,0x6B,
		0x6D,0x6E,0x70,0x71,0x72,0x74,0x75,0x77,0x78,0x7A,0x7B,0x7D,0x7E,0x80,0x82,0x83,
		0x85,0x86,0x88,0x8A,0x8B,0x8D,0x8F,0x90,0x92,0x94,0x95,0x97,0x99,0x9A,0x9C,0x9E,
		0xA0,0xA1,0xA3,0xA5,0xA7,0xA9,0xAA,0xAC,0xAE,0xB0,0xB2,0xB4,0xB6,0xB7,0xB9,0xBB,
		0xBD,0xBF,0xC1,0xC3,0xC5,0xC7,0xC9,0xCB,0xCD,0xCF,0xD1,0xD3,0xD5,0xD7,0xDA,0xDC,
		0xDE,0xE0,0xE2,0xE4,0xE6,0xE9,0xEB,0xED,0xEF,0xF1,0xF4,0xF6,0xF8,0xFA,0xFD,0xFF
	},

	/* Gamma = 2.4 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,
		0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x04,
		0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x08,0x08,0x08,0x09,0x09,
		0x09,0x0A,0x0A,0x0A,0x0B,0x0B,0x0B,0x0C,0x0C,0x0D,0x0D,0x0E,0x0E,0x0E,0x0F,0x0F,
		0x10,0x10,0x11,0x11,0x12,0x12,0x13,0x13,0x14,0x14,0x15,0x16,0x16,0x17,0x17,0x18,
		0x18,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,0x20,0x20,0x21,0x22,0x23,
		0x23,0x24,0x25,0x26,0x27,0x27,0x28,0x29,0x2A,0x2B,0x2B,0x2C,0x2D,0x2E,0x2F,0x30,
		0x31,0x32,0x33,0x34,0x35,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3E,0x3F,0x40,
		0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x50,0x51,0x52,
		0x53,0x55,0x56,0x57,0x58,0x5A,0x5B,0x5C,0x5E,0x5F,0x60,0x62,0x63,0x64,0x66,0x67,
		0x69,0x6A,0x6C,0x6D,0x6F,0x70,0x72,0x73,0x75,0x76,0x78,0x79,0x7B,0x7C,0x7E,0x7F,
		0x81,0x83,0x84,0x86,0x88,0x89,0x8B,0x8D,0x8E,0x90,0x92,0x94,0x95,0x97,0x99,0x9B,
		0x9C,0x9E,0xA0,0xA2,0xA4,0xA6,0xA7,0xA9,0xAB,0xAD,0xAF,0xB1,0xB3,0xB5,0xB7,0xB9,
		0xBB,0xBD,0xBF,0xC1,0xC3,0xC5,0xC7,0xC9,0xCB,0xCD,0xCF,0xD2,0xD4,0xD6,0xD8,0xDA,
		0xDC,0xDF,0xE1,0xE3,0xE5,0xE8,0xEA,0xEC,0xEF,0xF1,0xF3,0xF6,0xF8,0xFA,0xFD,0xFF
	},

	/* Gamma = 2.5 */
	{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,
		0x01,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x04,0x04,
		0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,
		0x08,0x08,0x09,0x09,0x09,0x0A,0x0A,0x0A,0x0B,0x0B,0x0C,0x0C,0x0C,0x0D,0x0D,0x0E,
		0x0E,0x0F,0x0F,0x0F,0x10,0x10,0x11,0x11,0x12,0x12,0x13,0x13,0x14,0x14,0x15,0x16,
		0x16,0x17,0x17,0x18,0x19,0x19,0x1A,0x1A,0x1B,0x1C,0x1C,0x1D,0x1E,0x1E,0x1F,0x20,
		0x21,0x21,0x22,0x23,0x24,0x24,0x25,0x26,0x27,0x28,0x28,0x29,0x2A,0x2B,0x2C,0x2D,
		0x2E,0x2E,0x2F,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,
		0x3D,0x3E,0x3F,0x40,0x41,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4B,0x4C,0x4D,0x4E,
		0x50,0x51,0x52,0x53,0x55,0x56,0x57,0x59,0x5A,0x5B,0x5D,0x5E,0x5F,0x61,0x62,0x63,
		0x65,0x66,0x68,0x69,0x6B,0x6C,0x6E,0x6F,0x71,0x72,0x74,0x75,0x77,0x79,0x7A,0x7C,
		0x7D,0x7F,0x81,0x82,0x84,0x86,0x87,0x89,0x8B,0x8D,0x8E,0x90,0x92,0x94,0x96,0x97,
		0x99,0x9B,0x9D,0x9F,0xA1,0xA3,0xA5,0xA6,0xA8,0xAA,0xAC,0xAE,0xB0,0xB2,0xB4,0xB6,
		0xB8,0xBA,0xBD,0xBF,0xC1,0xC3,0xC5,0xC7,0xC9,0xCC,0xCE,0xD0,0xD2,0xD4,0xD7,0xD9,
		0xDB,0xDD,0xE0,0xE2,0xE4,0xE7,0xE9,0xEB,0xEE,0xF0,0xF3,0xF5,0xF8,0xFA,0xFD,0xFF
	}
};

/****************************************************************************/

#endif /* CONFIG_SUPPORTS_COLOR */
